(**

  This module contains a class which represents a dynamic collection of process
  information.

  @Author  David Hoyle
  @Version 1.0
  @Date    01 May 2013

**)
Unit ExternalProcessInfo;

Interface

Uses
  DGHLibrary,
  INIFiles;

Type
  (** A class to represent a collection of TProcessInfo classes. **)
  TProcessCollection = Class
  Private
    FProcesses : Array Of TProcessInfo;
    FCount     : Integer;
  Protected
    Function  GetProcess(iIndex : Integer) : TProcessInfo;
    Procedure SetProcess(iIndex : Integer; PInfo : TProcessInfo);
  Public
    Constructor Create;
    Destructor Destroy; Override;
    Procedure AddProcessInfo(boolEnabled : Boolean; strEXE, strParams, strDir,
      strTitle : String);
    Procedure Clear;
    Procedure LoadFromINI(iniFile : TMemIniFile; strSection : String);
    Procedure SaveToINI(iniFile : TMemIniFile;strSection : String);
    (**
      A property to return the number of processes in the collection.
      @precon  None.
      @postcon Returns the numnber of process in the collection.
      @return  an Integer
    **)
    Property Count : Integer Read FCount;
    (**
      A property to return a reference to a process info record.
      @precon  iIndex must be a valid index in the collection.
      @postcon Returns a reference to the indexed process info record.
      @param   iIndex as       an Integer
      @return  a TProcessInfo
    **)
    Property Process[iIndex : Integer] : TProcessInfo Read GetProcess Write SetProcess;
      Default;
  End;

Implementation

Uses
  Classes,
  SysUtils;

Const
  (** A constant to define the growth capacity of the collection. **)
  iCAPACITY : Integer = 10;

{ TProcessCollection }

(**

  This method adds a set of process information to the collection, and grows the
  collection if necessary.

  @precon  None.
  @postcon Adds a set of process information to the collection, and grows the collection
           if neceaary.

  @param   boolEnabled as a Boolean
  @param   strEXE      as a String
  @param   strParams   as a String
  @param   strDir      as a String
  @param   strTitle    as a String

**)
procedure TProcessCollection.AddProcessInfo(boolEnabled: Boolean; strEXE,
  strParams, strDir, strTitle: String);

Var
  tmpProcesses : Array Of TProcessInfo;
  i : Integer;

begin
  If strEXE <> '' Then
    Begin
      Inc(FCount);
      If FCount > High(FProcesses) + 1 Then
        Begin
          SetLength(tmpProcesses, Length(FProcesses));
          For i := Low(FProcesses) To High(FProcesses) Do
            tmpProcesses[i] := FProcesses[i];
          SetLength(FProcesses, iCAPACITY + Length(FProcesses));
          For i := Low(tmpProcesses) to High(tmpProcesses) Do
            FProcesses[i] := tmpProcesses[i];
        End;
      FProcesses[FCount - 1].boolEnabled := boolEnabled;
      FProcesses[FCount - 1].strEXE := strEXE;
      FProcesses[FCount - 1].strParams := strParams;
      FProcesses[FCount - 1].strDir := strDir;
      FProcesses[FCount - 1].strTitle := strTitle;
    End;
end;

(**

  This method clears the existing collection and creates a new empty collection.

  @precon  None.
  @postcon Clears the existing collection and creates a new empty collection.

**)
procedure TProcessCollection.Clear;
begin
  FProcesses := Nil;
  SetLength(FProcesses, iCAPACITY);
  FCount := 0;
end;

(**

  This is the constructor method for the TProcessCollection class.

  @precon  None.
  @postcon Creates the empty collection.

**)
constructor TProcessCollection.Create;
begin
  SetLength(FProcesses, iCAPACITY);
  FCount := 0;
end;

(**

  This is the destructor method for the TProcessCollection class.

  @precon  None.
  @postcon Clears the collection.

**)
destructor TProcessCollection.Destroy;
begin
  FProcesses := Nil;
  Inherited;
end;

(**

  This is a getter method for the Process property.

  @precon  iIndex must eb a valid index in the collection.
  @postcon Returns the reference to the indexed procress info record.

  @param   iIndex as an Integer
  @return  a TProcessInfo

**)
function TProcessCollection.GetProcess(iIndex: Integer): TProcessInfo;
begin
  Result := FProcesses[iIndex];
end;

(**

  This method clears the current collection and loads a new set of process information
  from the inifile and section provided.

  @precon  iniFile must be a valid instance.
  @postcon Clears the current collection and loads a new set of process information from
           the inifile and section provided.

  @param   iniFile    as a TMemIniFile
  @param   strSection as a String

**)
procedure TProcessCollection.LoadFromINI(iniFile : TMemIniFile; strSection: String);

Var
  sl : TStringList;
  i : Integer;
  strTitle: String;

begin
  Clear;
  sl := TStringList.Create;
  Try
    iniFile.ReadSection(strSection, sl);
    For i := 0 To sl.Count - 1 Do
      Begin
        strTitle := GetField(sl[i], '|', 4);
        If strTitle = '' Then
          strTitle := ExtractFilename(GetField(sl[i], '|', 1));
        AddProcessInfo(
          iniFile.ReadBool(strSection, sl[i], True),
          GetField(sl[i], '|', 1),
          GetField(sl[i], '|', 2),
          GetField(sl[i], '|', 3),
          strTitle
        );
      End;
  Finally
    sl.Free;
  End;
end;

(**

  This method saves the collection of process information recofrds to the ini file and
  section provided.

  @precon  iniFile must be a valid instance.
  @postcon Saves the collection of process information recofrds to the ini file and
           section provided.

  @param   iniFile    as a TMemIniFile
  @param   strSection as a String

**)
procedure TProcessCollection.SaveToINI(iniFile : TMemIniFile; strSection: String);

Var
  i : Integer;

begin
  iniFile.EraseSection(strSection);
  For i := 0 To FCount - 1 Do
    iniFile.WriteBool(strSection, Format('%s|%s|%s|%s', [
      FProcesses[i].strEXE,
      FProcesses[i].strParams,
      FProcesses[i].strDir,
      FProcesses[i].strTitle
    ]), FProcesses[i].boolEnabled);
  iniFile.UpdateFile;
end;

(**

  This is a setter method for the Process property.

  @precon  iIndex must be a valid index between 0 and Count - 1.
  @postcon Updates the indexed process with the passed information.

  @param   iIndex as an Integer
  @param   PInfo  as a TProcessInfo

**)
Procedure TProcessCollection.SetProcess(iIndex: Integer; PInfo: TProcessInfo);

Begin
  FProcesses[iIndex].boolEnabled := PInfo.boolEnabled;
  FProcesses[iIndex].strEXE := PInfo.strEXE;
  FProcesses[iIndex].strParams := PInfo.strParams;
  FProcesses[iIndex].strDir := PInfo.strDir;
  FProcesses[iIndex].strTitle := PInfo.strTitle;
End;

End.
